#include <stdio.h>
#include "DM9000A.H"
#include "sys/alt_irq.h"

#include <io.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "system.h"
#include "sys/alt_irq.h"

void			iow(unsigned int reg, unsigned int data);
unsigned int	ior(unsigned int reg);
void			DM9000_init (void);
void			TransmitPacket(unsigned char *data_ptr,unsigned int tx_len);
void			ReceivePacket (unsigned char *data_ptr,unsigned int *rx_len);
void 			DM9000A_ISR(void* context, alt_u32 id);

unsigned char	rx_buffer[MAX_PACKET_SIZE];
unsigned int	rx_length; 	
//-----------------------------------------------------------------------------
unsigned char ether_addr[6]={ 0x01, 0x60, 0x6E, 0x11, 0x02, 0x0F  };

//-----------------------------------------------------------------------------
// Basic write access to DM9000A: writes 16 bits of "data" to register "reg".
void iow(unsigned int reg, unsigned int data)
{
	IOWR_16DIRECT(DM9000A_BASE, IO_addr, reg);		// write access to index port
	IOWR_16DIRECT(DM9000A_BASE, IO_data , data);	// write access to data port
}

//-----------------------------------------------------------------------------
// Basic read access to DM9000A: reads 16 bits from register "reg".
unsigned int ior(unsigned int reg)
{
	IOWR_16DIRECT(DM9000A_BASE, IO_addr, reg);		// write access to index port 
	return IORD_16DIRECT(DM9000A_BASE, IO_data );	// read access to data port
}

//-----------------------------------------------------------------------------
// DM9000_init routine
void DM9000_init(void)
{
	int i;
	
	// 1. power on internal PHY
	iow(GPR, 0x00);					// activate internal PHY
	msleep(5);						// wait > 2 ms for PHY power-up ready
	
	// 2. software reset 
	iow(NCR, 0x03);					// reset on, MAC loopback on
	msleep(5);						// wait > 10us for a software-RESET ok
	iow(NCR, 0x00);					// MAC loopback off
	
	// 3. clear Network Status Register(NSR) by reading out
	ior(NSR);

	// 4. set MAC address
	for (i=0; i<6; i++) { iow(PAR+i, ether_addr[i]); }
	
	// 5. set hash table
	//for(i=0; i<8; i++) { iow(MAR+i, 0xff); }
	
	// 6. clear Interrupt Status Register (NSR)
	iow(ISR, 0x00);
		
	// 7. register interrupt handler
	alt_irq_register(DM9000A_IRQ, 0, DM9000A_ISR); 

	// 8. enable RX interrupt & pointer auto return function
	iow(IMR, 0x81);

	// 9. enable RX & multicast reception
	iow(RCR , 0x09);

	iow(TCR2, 0x80);				// Switch LED to mode 1
} 

//-----------------------------------------------------------------------------
// Transmit one Packet TX I/O routine
void TransmitPacket(unsigned char *data_ptr, unsigned int tx_len)
{
	unsigned int  i;
	// issue TX packet's length into TXPLH REG. FDH & TXPLL REG. FCH
	iow(TXPLH, (tx_len >> 8) & 0xFF);		// TXPLH High_byte length
	iow(TXPLL, tx_len & 0xFF);				// TXPLL Low_byte  length
	// write transmit data to chip SRAM
	IOWR_16DIRECT(DM9000A_BASE, IO_addr, MWCMD);	// set MWCMD REG. F8H TX I/O port ready
	for (i=0; i<tx_len; i+=2)
	{
		IOWR_16DIRECT(DM9000A_BASE, IO_data, (data_ptr[i+1]<<8)|data_ptr[i]);
	}

	// issue TX polling command activated
	iow(TCR , 0x01);						// TXCR Bit [0] TXREQ auto clear after TX completed
}

//-----------------------------------------------------------------------------
// Receive one packet I/O routine
// If successfull reception, rx_len =! 0, and data_ptr cointains packet data
void ReceivePacket (unsigned char *data_ptr,unsigned int *rx_len)
{
	unsigned char rx_READY, GoodPacket;
	unsigned int  temp, RxStatus, i;
  
	IOWR_16DIRECT(DM9000A_BASE, IO_addr, MRCMDX);		// write access to index port 
	usleep(1);
	rx_READY = IORD_16DIRECT(DM9000A_BASE, IO_data);	// got most updated data
	usleep(1);
	rx_READY = IORD_16DIRECT(DM9000A_BASE, IO_data) & 0x03;	// re-read
  
	if (rx_READY == 0x01)					// received packet ready?
	{
		// got RX_Status & RX_Length from RX SRAM
		IOWR_16DIRECT(DM9000A_BASE, IO_addr, MRCMD);	// set DATA I/O mode
		usleep(1);
		RxStatus = IORD_16DIRECT(DM9000A_BASE,IO_data);	// the high byte is the status
		usleep(1);

		(*rx_len) = IORD_16DIRECT(DM9000A_BASE, IO_data);

		// Check this packet_status GOOD or BAD?
		if(!(RxStatus & 0xBF00) && ((*rx_len) <= MAX_PACKET_SIZE) && ((*rx_len) >= MIN_PACKET_SIZE))
		{
			GoodPacket=TRUE;
		}
		else
		{
			GoodPacket = FALSE;
			printf("\nError\n");
		}

		// read 1 received packet from RX SRAM into RX buffer
		for (i=0; i<(*rx_len); i+=2)
		{
			temp = IORD_16DIRECT(DM9000A_BASE, IO_data);

			if(GoodPacket)
			{
				data_ptr[i] = temp & 0xFF;	// good packet, save packet data
				data_ptr[i+1] = (temp>>8) & 0xFF;
			}
			else {}		// this packet is bad, dump it from RX SRAM
		}
		if(!GoodPacket) { (*rx_len) = 0; }
	}
	else if(rx_READY)	// hardware error: rx_READY Bit[1:0] must be "00"b or "01"b
	{
		DM9000_init();						// recover the unstable states by reinitialisation
		(*rx_len) = 0;
	}
}

//-----------------------------------------------------------------------------
// DM9000A Interrupt Service Routine
void DM9000A_ISR(void* context, alt_u32 id)
{
	unsigned char index_backup;
	
	// save current index port value
	index_backup = IORD_16DIRECT(DM9000A_BASE, IO_addr);

	ior(ISR);						// read/clear interrupt status register
	iow(ISR, 0x01);

	ReceivePacket(rx_buffer, &rx_length);	// receive packet
	if(rx_length)
	{
		TransmitPacket(rx_buffer, rx_length-4);	// send packet content unchanged back
	}

	// restore index register
	IOWR_16DIRECT(DM9000A_BASE, IO_addr, index_backup);
}
